@@ -28,6 +28,8 @@ TEST(aes_xts, alloc)
 
 	TESTLIB_ENV_AES_KEY_CHECK();
 
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
 	rc = zpc_aes_xts_alloc(NULL);
 	EXPECT_EQ(rc, ZPC_ERROR_ARG1NULL);
 
@@ -51,6 +53,8 @@ TEST(aes_xts, free)
 
 	TESTLIB_ENV_AES_KEY_CHECK();
 
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
 	zpc_aes_xts_free(NULL);
 
 	aes_xts = NULL;
@@ -73,12 +77,21 @@ TEST(aes_xts, set_key)
 	int rc, size, type;
 
 	TESTLIB_ENV_AES_KEY_CHECK();
+
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	TESTLIB_AES_XTS_KEY_SIZE_CHECK(size);
+
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -166,12 +179,21 @@ TEST(aes_xts, set_iv)
 	unsigned int flags;
 
 	TESTLIB_ENV_AES_KEY_CHECK();
+
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	TESTLIB_AES_XTS_KEY_SIZE_CHECK(size);
+
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -250,12 +272,21 @@ TEST(aes_xts, encrypt)
 	int rc, size, type;
 
 	TESTLIB_ENV_AES_KEY_CHECK();
+
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	TESTLIB_AES_XTS_KEY_SIZE_CHECK(size);
+
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -321,12 +352,21 @@ TEST(aes_xts, decrypt)
 	int rc, size, type;
 
 	TESTLIB_ENV_AES_KEY_CHECK();
+
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	TESTLIB_AES_XTS_KEY_SIZE_CHECK(size);
+
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -392,12 +432,21 @@ TEST(aes_xts, pc)
 	int rc, size, type;
 
 	TESTLIB_ENV_AES_KEY_CHECK();
+
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size = testlib_env_aes_key_size();
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	TESTLIB_AES_XTS_KEY_SIZE_CHECK(size);
+
 	memcpy(m_bak, m, 96);
 
 	rc = zpc_aes_key_alloc(&aes_key1);
@@ -534,6 +583,10 @@ TEST(aes_xts, stream_inplace_kat1)
 {
 	TESTLIB_ENV_AES_KEY_CHECK();
 
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size_t keylen, msglen, ctlen, ivlen;
 	unsigned char buf[4096];
 	const char *mkvp, *apqns[257];
@@ -547,6 +600,13 @@ TEST(aes_xts, stream_inplace_kat1)
 	const char *msgstr = "ec36551c70efcdf85de7a39988978263ad261e83996dad219a0058e02187384f2d0754ff9cfa000bec448fafd2cfa738";
 	const char *ctstr = "a55d533c9c5885562b92d4582ea69db8e2ba9c0b967a9f0167700b043525a47bafe7d630774eaf4a1dc9fbcf94a1fda4";
 
+	type = testlib_env_aes_key_type();
+	flags = testlib_env_aes_key_flags();
+	mkvp = testlib_env_aes_key_mkvp();
+	(void)testlib_env_aes_key_apqns(apqns);
+
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
 	u8 *key1 = testlib_hexstr2buf(keystr, &keylen);
 	ASSERT_NE(key1, nullptr);
 	keylen /= 2;
@@ -558,11 +618,6 @@ TEST(aes_xts, stream_inplace_kat1)
 	u8 *ct = testlib_hexstr2buf(ctstr, &ctlen);
 	ASSERT_NE(ct, nullptr);
 
-	type = testlib_env_aes_key_type();
-	flags = testlib_env_aes_key_flags();
-	mkvp = testlib_env_aes_key_mkvp();
-	(void)testlib_env_aes_key_apqns(apqns);
-
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -573,9 +628,9 @@ TEST(aes_xts, stream_inplace_kat1)
 	rc = zpc_aes_key_set_type(aes_key1, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key1, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -589,9 +644,9 @@ TEST(aes_xts, stream_inplace_kat1)
 	rc = zpc_aes_key_set_type(aes_key2, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key2, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -675,6 +730,10 @@ TEST(aes_xts, stream_inplace_kat2)
 {
 	TESTLIB_ENV_AES_KEY_CHECK();
 
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size_t keylen, ivlen, msglen, ctlen;
 	unsigned char buf[4096];
 	const char *mkvp, *apqns[257];
@@ -688,6 +747,13 @@ TEST(aes_xts, stream_inplace_kat2)
 	const char *msgstr = "2f3b9dcfbae729583b1d1ffdd16bb6fe2757329435662a78f0";
 	const char *ctstr = "f3473802e38a3ffef4d4fb8e6aa266ebde553a64528a06463e";
 
+	type = testlib_env_aes_key_type();
+	flags = testlib_env_aes_key_flags();
+	mkvp = testlib_env_aes_key_mkvp();
+	(void)testlib_env_aes_key_apqns(apqns);
+
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
 	u8 *key1 = testlib_hexstr2buf(keystr, &keylen);
 	ASSERT_NE(key1, nullptr);
 	keylen /= 2;
@@ -699,11 +765,6 @@ TEST(aes_xts, stream_inplace_kat2)
 	u8 *ct = testlib_hexstr2buf(ctstr, &ctlen);
 	ASSERT_NE(ct, nullptr);
 
-	type = testlib_env_aes_key_type();
-	flags = testlib_env_aes_key_flags();
-	mkvp = testlib_env_aes_key_mkvp();
-	(void)testlib_env_aes_key_apqns(apqns);
-
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -714,9 +775,9 @@ TEST(aes_xts, stream_inplace_kat2)
 	rc = zpc_aes_key_set_type(aes_key1, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key1, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -730,9 +791,9 @@ TEST(aes_xts, stream_inplace_kat2)
 	rc = zpc_aes_key_set_type(aes_key2, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key2, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -745,7 +806,7 @@ TEST(aes_xts, stream_inplace_kat2)
 
 	rc = zpc_aes_xts_set_key(aes_xts, aes_key1, aes_key2);
 	EXPECT_EQ(rc, 0);
-   
+
 	/* Encrypt */
 	memcpy(buf, msg, msglen);
 
@@ -784,6 +845,11 @@ TEST(aes_xts, stream_inplace_kat2)
 TEST(aes_xts, nist_kat)
 {
 	TESTLIB_ENV_AES_KEY_CHECK();
+
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	__run_json("nist_aes_xts.json");
 }
 
@@ -807,6 +873,8 @@ static void __run_json(const char *json)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -817,9 +885,9 @@ static void __run_json(const char *json)
 	rc = zpc_aes_key_set_type(aes_key1, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key1, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -829,9 +897,9 @@ static void __run_json(const char *json)
 	rc = zpc_aes_key_set_type(aes_key2, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key2, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -947,8 +1015,15 @@ TEST(aes_xts, rederive_protected_key1)
 	int rc, size;
 
 	TESTLIB_ENV_AES_KEY_CHECK();
+
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size = testlib_env_aes_key_size();
 
+	TESTLIB_AES_XTS_KEY_SIZE_CHECK(size);
+
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -1020,6 +1095,10 @@ TEST(aes_xts, rederive_protected_key2)
 {
 	TESTLIB_ENV_AES_KEY_CHECK();
 
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size_t keylen, msglen, ctlen, ivlen;
 	unsigned char buf[4096];
 	const char *mkvp, *apqns[257];
@@ -1033,6 +1112,13 @@ TEST(aes_xts, rederive_protected_key2)
 	const char *msgstr = "2f3b9dcfbae729583b1d1ffdd16bb6fe2757329435662a78f0";
 	const char *ctstr = "f3473802e38a3ffef4d4fb8e6aa266ebde553a64528a06463e";
 
+	type = testlib_env_aes_key_type();
+	flags = testlib_env_aes_key_flags();
+	mkvp = testlib_env_aes_key_mkvp();
+	(void)testlib_env_aes_key_apqns(apqns);
+
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
 	u8 *key1 = testlib_hexstr2buf(keystr, &keylen);
 	ASSERT_NE(key1, nullptr);
 	keylen /= 2;
@@ -1044,11 +1130,6 @@ TEST(aes_xts, rederive_protected_key2)
 	u8 *ct = testlib_hexstr2buf(ctstr, &ctlen);
 	ASSERT_NE(ct, nullptr);
 
-	type = testlib_env_aes_key_type();
-	flags = testlib_env_aes_key_flags();
-	mkvp = testlib_env_aes_key_mkvp();
-	(void)testlib_env_aes_key_apqns(apqns);
-
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -1059,9 +1140,9 @@ TEST(aes_xts, rederive_protected_key2)
 	rc = zpc_aes_key_set_type(aes_key1, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key1, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -1077,9 +1158,9 @@ TEST(aes_xts, rederive_protected_key2)
 	rc = zpc_aes_key_set_type(aes_key2, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key2, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -1165,6 +1246,10 @@ TEST(aes_xts, reencipher)
 {
 	TESTLIB_ENV_AES_KEY_CHECK();
 
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size_t keylen, msglen, ctlen, ivlen;
 	unsigned char buf[4096];
 	const char *mkvp, *apqns[257];
@@ -1194,6 +1279,10 @@ TEST(aes_xts, reencipher)
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	TESTLIB_AES_NEW_MK_CHECK(type, mkvp, apqns);
+
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -1204,9 +1293,9 @@ TEST(aes_xts, reencipher)
 	rc = zpc_aes_key_set_type(aes_key1, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key1, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -1222,9 +1311,9 @@ TEST(aes_xts, reencipher)
 	rc = zpc_aes_key_set_type(aes_key2, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key2, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -1395,6 +1484,10 @@ TEST(aes_xts, threads)
 {
 	TESTLIB_ENV_AES_KEY_CHECK();
 
+	TESTLIB_AES_XTS_HW_CAPS_CHECK();
+
+	TESTLIB_AES_KERNEL_CAPS_CHECK();
+
 	size_t keylen;
 	const char *mkvp, *apqns[257];
 	struct zpc_aes_key *aes_key1, *aes_key2;
@@ -1404,15 +1497,19 @@ TEST(aes_xts, threads)
 
 	const char *keystr = "394c97881abd989d29c703e48a72b397a7acf51b59649eeea9b33274d8541df4";
 
-	u8 *key1 = testlib_hexstr2buf(keystr, &keylen);
-	keylen /= 2;
-	u8 *key2 = key1 + keylen;
-
 	type = testlib_env_aes_key_type();
 	flags = testlib_env_aes_key_flags();
 	mkvp = testlib_env_aes_key_mkvp();
 	(void)testlib_env_aes_key_apqns(apqns);
 
+	TESTLIB_AES_SW_CAPS_CHECK(type);
+
+	TESTLIB_AES_NEW_MK_CHECK(type, mkvp, apqns);
+
+	u8 *key1 = testlib_hexstr2buf(keystr, &keylen);
+	keylen /= 2;
+	u8 *key2 = key1 + keylen;
+
 	rc = zpc_aes_key_alloc(&aes_key1);
 	EXPECT_EQ(rc, 0);
 	rc = zpc_aes_key_alloc(&aes_key2);
@@ -1421,9 +1518,9 @@ TEST(aes_xts, threads)
 	rc = zpc_aes_key_set_type(aes_key1, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key1, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key1, apqns);
 		EXPECT_EQ(rc, 0);
 	}
@@ -1439,9 +1536,9 @@ TEST(aes_xts, threads)
 	rc = zpc_aes_key_set_type(aes_key2, type);
 	EXPECT_EQ(rc, 0);
 	if (mkvp != NULL) {
-		  rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
-		  EXPECT_EQ(rc, 0);
-	 } else {
+		rc = zpc_aes_key_set_mkvp(aes_key2, mkvp);
+		EXPECT_EQ(rc, 0);
+	} else {
 		rc = zpc_aes_key_set_apqns(aes_key2, apqns);
 		EXPECT_EQ(rc, 0);
 	}
